unit Main;
////////////////////////////////////////////////////////////////////////////////
///
///               DEMO APPLICATION FOR PRG07N.dll LIBRARY
///
///  Application don't cover usage of all functions PRG07N.dll. It only
///  show common metodology for DLL usage.
///  This is 32-bit application, and use 32-bit version of DLL. In SDK
///  you can find both versions of DLL - for 32 and 64 bit Windows.
///
////////////////////////////////////////////////////////////////////////////////


interface
////////////////////////////////////////////////////////////////////////////////
uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.Mask, Vcl.ExtCtrls;

type
  TfrmMain = class(TForm)
    sbrStatus: TStatusBar;
    pgcPages: TPageControl;
    tbsStep1: TTabSheet;
    edtIP: TMaskEdit;
    Label2: TLabel;
    edtSrv: TMaskEdit;
    Label4: TLabel;
    Label1: TLabel;
    btnDoStep1: TBitBtn;
    tbsStep2: TTabSheet;
    tbsStep3: TTabSheet;
    Label3: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    btnReadVersion: TBitBtn;
    btnSetupRtc: TBitBtn;
    btnGoStep3: TBitBtn;
    edtReaderVersion: TEdit;
    btnReadSerial: TBitBtn;
    edtReaderSerial: TEdit;
    lblSettingDateTime: TLabel;
    btnGoStep4: TBitBtn;
    Label10: TLabel;
    Bevel1: TBevel;
    Label8: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    cbxChanActive1: TCheckBox;
    udMemTime1: TUpDown;
    edtMemTimeMon1: TEdit;
    edtDumpinMon1: TEdit;
    udDumping1: TUpDown;
    Bevel2: TBevel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    cbxChanActive2: TCheckBox;
    udMemTime2: TUpDown;
    edtMemTimeMon2: TEdit;
    edtDumpinMon2: TEdit;
    udDumping2: TUpDown;
    btnWriteSettings: TBitBtn;
    btnReadSettings: TBitBtn;
    tnsStep4: TTabSheet;
    btnFinish: TBitBtn;
    lvAudit: TListView;
    cbxClearReaderTransactions: TCheckBox;
    btnClearAudit: TBitBtn;
    Panel1: TPanel;
    Label9: TLabel;
    procedure btnDoStep1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnReadVersionClick(Sender: TObject);
    procedure btnSetupRtcClick(Sender: TObject);
    procedure btnReadSerialClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnGoStep3Click(Sender: TObject);
    procedure tbsStep3Enter(Sender: TObject);
    procedure btnReadSettingsClick(Sender: TObject);
    procedure btnWriteSettingsClick(Sender: TObject);
    procedure btnGoStep4Click(Sender: TObject);
    procedure tnsStep4Enter(Sender: TObject);
    procedure tnsStep4Exit(Sender: TObject);
    procedure btnClearAuditClick(Sender: TObject);
    procedure btnFinishClick(Sender: TObject);
  private
    { Private declarations }
    res: word;
    Step2Ver: boolean;
    Step2Serial: boolean;
    Step2DTime: boolean;
    StoredHandler: TIdleEvent;

    function ShowStatus(stat: word): word;
  public
    { Public declarations }
    procedure AuditHandler(Sender: TObject; var Done: Boolean);
  end;

var
  frmMain: TfrmMain;

implementation
////////////////////////////////////////////////////////////////////////////////
{$R *.dfm}
uses
  RUtils,
  Prg07Dll, Warning;

const
  DEF_READER_ADDR = 1;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  //
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.FormShow(Sender: TObject);
var
  n: integer;
begin
  if (frmWarning.ShowModal <> mrOk) then begin
    Application.Terminate;
  end;
  for n := 0 to pgcPages.PageCount - 1 do
    pgcPages.Pages[n].TabVisible := false;
  pgcPages.ActivePageIndex := 0;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Stop DLL before close application
  if (DllStarted > 0) then
    DllStop();
end;

////////////////////////////////////////////////////////////////////////////////
function TfrmMain.ShowStatus(stat: word): word;
begin
  // Show status of last operation
  sbrStatus.Panels[1].Text := 'Status: ' + ErrorCodeToStr(stat);
  Application.ProcessMessages;
  result := stat;
end;

////////////////////////////////////////////////////////////////////////////////
//         Show error message, stop DLL and terminate application
////////////////////////////////////////////////////////////////////////////////
procedure ErrorTerminate(msg: string);
begin
  ShowMessage('Error: ' + msg + '.' + ^M + ^J + 'Program will be terminated');
  DllStop;
  Application.Terminate;
end;

////////////////////////////////////////////////////////////////////////////////
//                                 STEP 1
//   1. Set IP-address for reader
//   2. Set correct host IP-address
//   3. Try to open DLL
//   4. If success - go to step 2
////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnDoStep1Click(Sender: TObject);
var
  tmp_ip: uint32;
begin
  // Set correct IP for DLL
  tmp_ip := StrToIp(edtIP.Text);
  res := SetReaderIp(tmp_ip);
  tmp_ip := StrToIp(edtSrv.Text);
  res := SetServerIp(tmp_ip);
  // Try to start DLL
  DllStart(1);
  // Look for result
  if (DllStarted > 0) then begin
    ShowStatus(RES_NO_ERROR);
    btnGoStep3.Enabled := false;
    //
    Step2Ver := false;
    Step2Serial := false;;
    Step2DTime := false;
    pgcPages.ActivePageIndex := pgcPages.ActivePageIndex + 1;
  end else begin
    ShowStatus(RES_INTERFACE_ERR);
  end;
end;


////////////////////////////////////////////////////////////////////////////////
//                                 STEP 2
//   1. Get reader virsion
//   2. Get reader serial number
//   3. Setup reader RTC
//   4. If success - go to step 3
////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnReadVersionClick(Sender: TObject);
var
  ver: array[0..127] of byte;
  s: AnsiString;
begin
  res := ShowStatus(GetVersionStr(DEF_READER_ADDR, @ver));
  if (res <> RES_NO_ERROR) then begin
    ErrorTerminate('Cannot get version');
  end else begin
    s := Buffer2String(@ver);
    edtReaderVersion.Text := s;
    // Check if we can go to next step
    Step2Ver := true;
    if ((Step2Ver) and (Step2Serial) and (Step2DTime)) then
      btnGoStep3.Enabled := true;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnReadSerialClick(Sender: TObject);
var
  ser: array[0..15] of byte;
  s: string;
begin
  res := ShowStatus(ReadSerial(DEF_READER_ADDR, @ser));
  if (res <> RES_NO_ERROR) then begin
    ErrorTerminate('Cannot read serial');
  end else begin
    s := SerialToStr(@ser);
    edtReaderSerial.Text := s;
    // Check if we can go to next step
    Step2Serial := true;
    if ((Step2Ver) and (Step2Serial) and (Step2DTime)) then
      btnGoStep3.Enabled := true;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnSetupRtcClick(Sender: TObject);
var
  dat, tim: uint32;
  s: string;
begin
  dat := DateToLPC(now);
  tim := TimeToLPC(now);
  res := ShowStatus(SetDateTime(DEF_READER_ADDR, tim, dat));
  if (res <> RES_NO_ERROR) then begin
    ErrorTerminate('Cannot setup RTC');
  end else begin
    s := 'Set to reader: ' + DateToStr(now) + '  ' + TimeToStr(now);
    lblSettingDateTime.Caption:= s;
    // Check if we can go to next step
    Step2DTime := true;
    if ((Step2Ver) and (Step2Serial) and (Step2DTime)) then
      btnGoStep3.Enabled := true;
  end;

end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnGoStep3Click(Sender: TObject);
begin
  // Go to next page (step)
  pgcPages.ActivePageIndex := pgcPages.ActivePageIndex + 1;
end;


////////////////////////////////////////////////////////////////////////////////
//                                 STEP 3
//   1. Read main reader settings and display it
//   2. User can change settings and write to reader
//   3. If success - go to step 3
////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.tbsStep3Enter(Sender: TObject);
begin
  btnReadSettingsClick(self);
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnReadSettingsClick(Sender: TObject);
var
  a1, a2, l1, l2: word;
  mt1, mt2: word;
  d1, d2: word;
begin
  // Get radio channels status
  if (ShowStatus(GetRfChannelsStat(DEF_READER_ADDR, a1, a2, l1, l2)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot get RC activity');
  // If OK - show activity on screen
  cbxChanActive1.Checked := (a1 <> 0);
  cbxChanActive2.Checked := (a2 <> 0);
  // Get tag memory times
  if (ShowStatus(GetTagMemTime(DEF_READER_ADDR, 1, mt1)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot get TagMemTime');
  if (ShowStatus(GetTagMemTime(DEF_READER_ADDR, 2, mt2)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot get TagMemTime');
  // If OK - show parameter on screen
  udMemTime1.Position := mt1 div 10;
  udMemTime2.Position := mt2 div 10;
  // Get dumping for channels
  if (ShowStatus(GetDumping(DEF_READER_ADDR, 1, d1)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot get dumping');
  if (ShowStatus(GetDumping(DEF_READER_ADDR, 2, d2)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot get dumping');
  // If OK - show parameter on screen
  udDumping1.Position := d1;
  udDumping2.Position := d2;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnWriteSettingsClick(Sender: TObject);
var
  a1, a2, l1, l2: word;
  mt1, mt2: word;
  d1, d2: word;
begin
  // Get parameters from screen
  a1 := 0; a2 := 0;
  if (cbxChanActive1.Checked) then
    a1 := 1;
  if (cbxChanActive2.Checked) then
    a2 := 1;
  mt1 := udMemTime1.Position;
  mt2 := udMemTime2.Position;
  d1 := udDumping1.Position;
  d2 := udDumping2.Position;
  // Try to write to reader
  if (ShowStatus(SetChannelActivity(DEF_READER_ADDR, 1, a1)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot set RC activity');
  if (ShowStatus(SetChannelActivity(DEF_READER_ADDR, 2, a2)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot set RC activity');
  if (ShowStatus(SetTagMemTime(DEF_READER_ADDR, 1, (mt1 * 10))) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot set TagMemTime');
  if (ShowStatus(SetTagMemTime(DEF_READER_ADDR, 2, (mt2 * 10))) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot set TagMemTime');
  if (ShowStatus(SetDumping(DEF_READER_ADDR, 1, d1)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot set dumping');
  if (ShowStatus(SetDumping(DEF_READER_ADDR, 2, d2)) <> RES_NO_ERROR) then
    ErrorTerminate('Cannot set dumping');
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnGoStep4Click(Sender: TObject);
begin
  // Go to next page (step)
  pgcPages.ActivePageIndex := pgcPages.ActivePageIndex + 1;
end;


////////////////////////////////////////////////////////////////////////////////
//                                 STEP 4
//   1. Real time audit
//   2. Finish
////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.tnsStep4Enter(Sender: TObject);
begin
  lvAudit.Columns[0].Width := 65;
  lvAudit.Columns[1].Width := 150;
  lvAudit.Columns[2].Width := 110;
  lvAudit.Columns[3].Width := 260;
  //
  StoredHandler := Application.OnIdle;
  Application.OnIdle := AuditHandler;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.tnsStep4Exit(Sender: TObject);
begin
  Application.OnIdle := nil;
  Application.OnIdle := StoredHandler;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnClearAuditClick(Sender: TObject);
begin
  lvAudit.Items.Clear;
  if (cbxClearReaderTransactions.Checked) then begin
    // Audit stop
    Application.OnIdle := StoredHandler;
    Sleep(100);
    ShowStatus(ClearTransactions(DEF_READER_ADDR));
    // Restart audit
    Sleep(100);
    Application.OnIdle := AuditHandler;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.AuditHandler(Sender: TObject; var Done: Boolean);
var
  res: word;
  tran: ttransaction;
  li: TListItem;
  Ts: string;
  Dt: TDateTime;
begin
  res := AskTransaction(DEF_READER_ADDR, tran);
  ShowStatus(res);
  if res = RES_NO_ERROR then begin
    if tran.code <> 0 then begin
      li := lvAudit.Items.Insert(0);
      // Channel number
      li.Caption := IntToStr(tran.srcnumber);
      // Date and time
      Dt := EncodeDate(tran.year, tran.month, tran.day);
      Ts := FormatDateTime('dd-mm-yy', dt) + ' ' +
        IntToStrLen(tran.hour, 2) + ':' +
                    IntToStrLen(tran.minute, 2) + ':' +
                    IntToStrLen(tran.sec, 2);
      li.SubItems.Add(Ts);
      // Tag ID (code)
      if (tran.card <> 0) then
        li.SubItems.Add('0x' + IntToHex(tran.card, 8))
      else
        li.SubItems.Add('');
      // Transaction text
      li.SubItems.Add(TransactionToStr(tran.code));
    end;
  end else begin

  end;
  Sleep(200);
  Done := false;
end;

////////////////////////////////////////////////////////////////////////////////
procedure TfrmMain.btnFinishClick(Sender: TObject);
begin
  // Close application
  Close;
end;

end.

