unit RUtils;

                            interface
////////////////////////////////////////////////////////////////////////////////
uses
  Controls,
  SysUtils;


function IntToStrLen(inp: integer; l: integer): string;
function HexTextToCode(s: string): uint32;
function DateTimeToLPC(dat: TDateTime): uint32;
function DateToLPC(dat: TDateTime): uint32;
function TimeToLPC(tim: TDateTime): uint32;
//
function IpArray2Word(ip: pbytearray): uint32;
function IpToStr(ip: uint32): string;
function StrToIp(s: string): uint32;
function StrToIpL(s: string): uint32;
function StrToMAC(strMAC: string; mac: pbytearray): boolean;
function MacToStr(mac: pbytearray): string;
//
function SerialToStr(ser: pbytearray): string;
function StrToSerial(ss: string; ser: pbytearray): boolean;
//
function BoolToInt(b: boolean): integer;
//
function TransactionToStr(code: word): string;
//
function ErrorCodeToStr(code: word): string;
//
function Buffer2String(src: PByteArray): AnsiString;


                         implementation
////////////////////////////////////////////////////////////////////////////////
uses
  PRG07Dll;

function IntToStrLen(inp: integer; l: integer): string;
var
  s: string;
begin
  s := IntToStr(inp);
  while (length(s) < l) do
    s := '0' + s;
  result := s;
end;

function MakeStr(C: Char; N: Integer): string;
begin
  if N < 1 then Result := ''
  else begin
{$IFNDEF WIN32}
    if N > 255 then N := 255;
{$ENDIF WIN32}
    SetLength(Result, N);
    FillChar(Result[1], Length(Result), C);
  end;
end;

function DelChars(const S: string; Chr: Char): string;
var
  I: Integer;
begin
  Result := S;
  for I := Length(Result) downto 1 do begin
    if Result[I] = Chr then Delete(Result, I, 1);
  end;
end;

function DelSpace(const S: String): string;
begin
  Result := DelChars(S, ' ');
end;

function DelBSpace(const S: string): string;
var
  I, L: Integer;
begin
  L := Length(S);
  I := 1;
  while (I <= L) and (S[I] = ' ') do Inc(I);
  Result := Copy(S, I, MaxInt);
end;

function Copy2Symb(const S: string; Symb: Char): string;
var
  P: Integer;
begin
  P := Pos(Symb, S);
  if P = 0 then P := Length(S) + 1;
  Result := Copy(S, 1, P - 1);
end;

function Copy2SymbDel(var S: string; Symb: Char): string;
begin
  Result := Copy2Symb(S, Symb);
  S := DelBSpace(Copy(S, Length(Result) + 1, Length(S)));
end;

function AddChar(C: Char; const S: string; N: Integer): string;
begin
  if Length(S) < N then
    Result := MakeStr(C, N - Length(S)) + S
  else Result := S;
end;


////////////////////////////////////////////////////////////////////////////////
//            (  )   
////////////////////////////////////////////////////////////////////////////////
function HexTextToCode(s: string): uint32;
var
  p: integer;
  d: integer;
begin
  //  :
  repeat
    p := Pos(' ', s);
    if p <> 0 then
      Delete(s, p, 1);
  until p = 0;
  //    :
  if TryStrToInt('$0' + s, d) then
    result := d
  else
    result := 0;
end;

////////////////////////////////////////////////////////////////////////////////
//            Delphi  ,   LPC20000
//  CTIME1   :
// 4:0 Day of Month Day of month value in the range of 1 to 28, 29, 30, or 31
// 7:5 - Reserved, user software should not write ones to reserved bits.
// 11:8 Month Month value in the range of 1 to 12.
// 15:12 - Reserved, user software should not write ones to reserved bits.
// 27:16 Year Year value in the range of 0 to 4095.
// 31:28 - Reserved, user software should not write ones to reserved bits.
////////////////////////////////////////////////////////////////////////////////
function DateTimeToLPC(dat: TDateTime): uint32;
var
  Year, Month, Day: word;
begin
  DecodeDate(dat, Year, Month, Day);
  result := day or (Month shl 8) or (Year shl 16);
end;

///////////////////////////////////////////////////////////////////////////////
//            Delphi  ,   LPC20000
//  CTIME1   :
// 4:0 Day of Month Day of month value in the range of 1 to 28, 29, 30, or 31
// 7:5 - Reserved, user software should not write ones to reserved bits.
// 11:8 Month Month value in the range of 1 to 12.
// 15:12 - Reserved, user software should not write ones to reserved bits.
// 27:16 Year Year value in the range of 0 to 4095.
// 31:28 - Reserved, user software should not write ones to reserved bits.
////////////////////////////////////////////////////////////////////////////////
function DateToLPC(dat: TDateTime): uint32;
var
  Year, Month, Day: word;
begin
  DecodeDate(dat, Year, Month, Day);
  result := day or (Month shl 8) or (Year shl 16);
end;

////////////////////////////////////////////////////////////////////////////////
//            Delphi  ,   LPC20000
////////////////////////////////////////////////////////////////////////////////
function TimeToLPC(tim: TDateTime): uint32;
var
  hour, min, sec, msec: word;
  dow: word;
begin
  DecodeTime(tim, hour, min, sec, msec);
  dow := DayOfWeek(tim);
  result := sec or (min shl 8) or (hour shl 16) or (dow shl 24);
end;

////////////////////////////////////////////////////////////////////////////////
function IpArray2Word(ip: pbytearray): uint32;
begin
  result := (ip^[0] shl 24) + (ip^[1] shl 16) + (ip^[2] shl 8) + ip^[3];
end;

////////////////////////////////////////////////////////////////////////////////
function IpToStr(ip: uint32): string;
begin
  result := IntTostr((ip shr 24) and $000000FF) + '.' +
            IntTostr((ip shr 16) and $000000FF) + '.' +
            IntTostr((ip shr 8) and $000000FF) + '.' +
            IntTostr(ip and $000000FF);
end;

////////////////////////////////////////////////////////////////////////////////
function StrToIp(s: string): uint32;
var
  ts: string;
  tdw: uint32;
begin
  s := DelSpace(s);
  tdw := 0;
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 24);
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 16);
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 8);
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 0);

  result := tdw;
end;

////////////////////////////////////////////////////////////////////////////////
function StrToIpL(s: string): uint32;
var
  ts: string;
  tdw: uint32;
begin
  s := DelSpace(s);
  tdw := 0;
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 0);
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 8);
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 16);
  ts := Copy2SymbDel(s, '.');
  Delete(s, 1, 1);
  tdw := tdw + uint32(StrToInt('0' + ts) shl 24);

  result := tdw;
end;


////////////////////////////////////////////////////////////////////////////////
function MacToStr(mac: pbytearray): string;
begin
  result := IntToHex(mac^[0], 2) + '-' +
            IntToHex(mac^[1], 2) + '-' +
            IntToHex(mac^[2], 2) + '-' +
            IntToHex(mac^[3], 2) + '-' +
            IntToHex(mac^[4], 2) + '-' +
            IntToHex(mac^[5], 2);
end;

////////////////////////////////////////////////////////////////////////////////
function StrToMAC(strMAC: string; mac: pbytearray): boolean;
var
  ts: string;
begin
  strMAC := DelSpace(strMAC);
  ts := Copy2SymbDel(strMAC, '-');
  Delete(strMAC, 1, 1);
  mac^[0] := StrToInt('$0' + ts);
  ts := Copy2SymbDel(strMAC, '-');
  Delete(strMAC, 1, 1);
  mac^[1] := StrToInt('$0' + ts);
  ts := Copy2SymbDel(strMAC, '-');
  Delete(strMAC, 1, 1);
  mac^[2] := StrToInt('$0' + ts);
  ts := Copy2SymbDel(strMAC, '-');
  Delete(strMAC, 1, 1);
  mac[3] := StrToInt('$0' + ts);
  ts := Copy2SymbDel(strMAC, '-');
  Delete(strMAC, 1, 1);
  mac^[4] := StrToInt('$0' + ts);
  ts := Copy2SymbDel(strMAC, '-');
  Delete(strMAC, 1, 1);
  mac^[5] := StrToInt('$0' + ts);

  result := true;
end;

(*
       ( 8 )  
  :
               7        6       5      4      3      2      1      0
            ID(h)  ID(lo)  VHard  VSoft    ----serial------
         BCD - 
*)
////////////////////////////////////////////////////////////////////////////////
//                
////////////////////////////////////////////////////////////////////////////////
function SerialToStr(ser: pbytearray): string;
var
  s: string;
begin
  s := IntToHex(ser[6], 1) + IntToHex((ser[5] + ser[4]), 2) + '-' +
       IntToHex(ser[3], 2) + '-' +
       IntToHex(ser[2], 2) + IntToHex(ser[1], 2) + IntToHex(ser[0], 2);
  result := s;
end;

////////////////////////////////////////////////////////////////////////////////
//                   
////////////////////////////////////////////////////////////////////////////////
function StrToSerial(ss: string; ser: pbytearray): boolean;
var
  c: string;

  function StrBcdDyte(s: string): byte;
  begin
    s := AddChar('0', s, 2);
    result := (StrToInt(s[1]) shl 4) + StrToInt(s[2]);
  end;

begin
  ser^[7] := 0;
  DelSpace(ss);
  //  ID 
  c := Copy2SymbDel(ss, '-');
  c := AddChar('0', c, 3);
  Delete(ss, 1, 1);
  ser^[6] := StrBcdDyte(Copy(c, 1, 1));
  Delete(c, 1, 1);
  //      
  ser^[5] := StrBcdDyte(Copy(c, 1, 1) + '0');
  //   
  c := AddChar('0', Copy(c, 2, 1), 2);
  ser^[4] := StrBcdDyte(c);
  //   
  c := Copy2SymbDel(ss, '-');
  c := AddChar('0', c, 2);
  Delete(ss, 1, 1);
  ser^[3] := StrBcdDyte(c);
  //   
  ss := AddChar('0', ss, 6);
  c := Copy(ss, 1, 2);
  ser^[2] := StrBcdDyte(c);
  c := Copy(ss, 3, 2);
  ser^[1] := StrBcdDyte(c);
  c := Copy(ss, 5, 2);
  ser^[0] := StrBcdDyte(c);

  result := true;
end;

function BoolToInt(b: boolean): integer;
begin
  if (b = true) then
    result := 1
  else
    result := 0;
end;

////////////////////////////////////////////////////////////////////////////////
function TransactionToStr(code: word): string;
begin
  case code of
    TRN_NEW_TAG_FOUND       : result := 'New tag found';
    TRN_TAG_LOSS            : result := 'Tag loss';
    TRN_DC_IS_ON            : result := 'Gate is on';
    TRN_DC_IS_OFF           : result := 'Gate is off';
    TRN_SENS_IS_ON          : result := 'Car sensor on';
    TRN_SENS_IS_OFF         : result := 'Car sensor off';
    TRN_SEND_TAG_1          : result := 'Tag send for entry';
    TRN_SEND_TAG_2          : result := 'Tag send for exit';
    TRN_LOCKED              : result := 'Locked';
    TRN_UNLOCKED            : result := 'Unlocked';
    TRN_SET_ON              : result := 'Set on';
    TRN_SET_OFF             : result := 'Set off';
    TRN_TAG_IN_OPPOSIT      : result := 'Tag in opposit';
    TRN_STACK_CLEARED       : result := 'Stack cleared';
    TRN_TAG_REMOVED         : result := 'Tag removed';
    TRN_WAIT_CAR            : result := 'Wait car';
    TRN_WAIT_ACCESS         : result := 'Wait access';
    TRN_WAIT_DEF_GATE       : result := 'Wait def gate time';
    TRN_WAIT_GATE_CLOSE     : result := 'Wait gate close';
    TRN_ACCESS_FINISH       : result := 'Access finished';
    TRN_CAR_NOT_FOUND       : result := 'Car not found';
    TRN_GATE_NOT_OPEN       : result := 'Gate not opened';
    TRN_GATE_NOT_CLOSE      : result := 'Gate not closed';
    TRN_WAIT_DEF_ACCESS     : result := 'Default access time';
    TRN_STACK_OVERFLOW      : result := 'Stack overflow';
    //
    TRN_BATT_LOW            : result := 'Batteri low';
    TRN_BATT_NORM           : result := 'Battery OK';
    TRN_CONTROLLER_ON       : result := 'Reader ON';
    TRN_CONTROLLER_OFF      : result := 'Reader off';

  else
    result := '';
  end;
  result := DelChars(result, '"');
end;

////////////////////////////////////////////////////////////////////////////////
function ErrorCodeToStr(code: word): string;
begin
  case (code) of
    RES_NO_ERROR:        result := 'OK';
    RES_INVALID_PARAM:   result := 'Invalid parameter';
    RES_INVALID_COMMAND: result := 'Invalid command';
    RES_BAD_CRC:         result := 'Invalid CRC';
    RES_TIMEOUT:         result := 'Exchange timeout';
    RES_INTERFACE_ERR:   result := 'Interface error';
    RES_NO_DATA:         result := 'No valid data';
  else
    result := 'UNKNOWN'
  end;
end;

////////////////////////////////////////////////////////////////////////////////
function Buffer2String(src: PByteArray): AnsiString;
var
 s: AnsiString;
 n: integer;
begin
  s := '';
  n := 0;
  while ((src^[n] <> 0) and (length(s) < 64)) do begin
    s := s + AnsiChar(src^[n]);
    inc(n);
  end;
  result := s;
end;

end.

